/*
John William Lynch
CSC 472 : Assignment 2
Multi Notes Program
Edit Activity, that contains the xml to add in json information to notes
 */

package com.example.a472_a2;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;

import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.EditText;
import android.widget.Toast;

public class EditActivity extends AppCompatActivity {

    //Create objects that we can review later:
    private MenuItem infoIcon;
    private MenuItem saveButton;
    EditText titleInput, noteInput;
    private static final String TAG = "EditActivity";

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_edit);
        setTitle("Multi Notes");
        invalidateOptionsMenu();

        titleInput = findViewById(R.id.noteTitle);
        noteInput = findViewById(R.id.noteInfo);
        noteInput.setMovementMethod(new ScrollingMovementMethod());

        //Double check the data to see if any changes have occured:
        //This will replace data so that it won't be empty once the user
        //does a single press on our assignment variables
        if (getIntent().getSerializableExtra("MainActivity") != null){
            Note n = (Note) getIntent().getSerializableExtra("MainActivity");
            titleInput.setText(n.getTitle());
            noteInput.setText(n.getContent());
        }


    }

    @Override
    protected  void onResume() {
        super.onResume();

    }

    /*
    Code that creates the action_menu so that icons can be displayed/selected
    */
    @Override
    public boolean onCreateOptionsMenu(Menu menu){

        getMenuInflater().inflate(R.menu.action_menu, menu);
        infoIcon = menu.findItem(R.id.informationIcon);
        infoIcon.setVisible(false);
        saveButton = menu.findItem(R.id.addItemIcon);
        saveButton.setVisible(true);

        return true;
    }

    /*
    We want to only show the save button inside of our code, not the help button as well
    so we can use set visible/invisible at this part to only show the certain item we want
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        /*
        Switch case, where the user presses the button again and intends to save both the note
        and the title: alongside the time:
         */
        switch (item.getItemId())   {
            case R.id.addItemIcon:
                //If the user enters in a blank title, we leave them
                if (!titleInput.getText().toString().isEmpty()){
                    /*
                    Since we are passing multiple values, we can try to bundle them over to the
                    next activity, as passing them in single putExtra lines causes null errors:
                     */
                    Intent intent = new Intent(EditActivity.this, MainActivity.class);
                    Bundle textSend = new Bundle();
                    Note note = new Note(titleInput.getText().toString(), noteInput.getText().toString(), " ");
                    textSend.putSerializable("EditActivity", note);
                    setResult(0,intent);
                    intent.putExtras(textSend);
                }
                else if (TextUtils.isEmpty(titleInput.getText().toString())){
                    Toast.makeText(getApplicationContext(), "SHORT " + "Please enter in a string title", Toast.LENGTH_SHORT).show();
                    finish();
                }
                finish();
                return true;
            case android.R.id.home:
                onBackPressed();
                return true;
                }
            return super.onOptionsItemSelected(item);
        }

    /*
    Create the function that will pass the data to a new intent that will then
    be passed over to the main function for note creation:
    We also want to check if the user has hit the back button during note creation:
    If the user has, we must prompt them to see if they have started creating a note via checking if
    either of the two note areas have been filled out, and then present them with a dialog choice
    to continue going back or saving the information:
    Below code taken from CatchBack code provided on D2L
     */


    @Override
    public void onBackPressed() {
        //Check to see if any data was entered before the back press, if not then just continue with
        //the back press:
        //If nothing is done, just exist back to first activity
        Log.d(TAG, "onBackPressed: " + titleInput.getText().toString());
        if (titleInput.getText().toString().isEmpty()) {
            finish();
            return;
            //Log.d(TAG, "onBackPressed: in first if statement" + titleInput.getText().toString());
            //finish();
            //return;
        }
        else if (TextUtils.isEmpty(titleInput.getText().toString()) && !TextUtils.isEmpty(noteInput.getText().toString())){
            Log.d(TAG, "onBackPressed: in second if statement" + titleInput.getText().toString());
            Toast.makeText(getApplicationContext(), "SHORT " + "No Information Added...Nothing Saved", Toast.LENGTH_SHORT).show();
            finish();
            return;
        }
        else {
            if (!titleInput.getText().toString().isEmpty()) {
                Log.d(TAG, "onBackPressed: in dialog box" + titleInput.getText().toString());
                AlertDialog.Builder builder = new AlertDialog.Builder(this);
                builder.setTitle("Save data");
                builder.setMessage("Your note is note saved!\n" + "Save note '" + titleInput.getText().toString() + "'?");
                builder.setPositiveButton("Yes", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        Intent intent = new Intent(EditActivity.this, MainActivity.class);
                        Bundle textSend = new Bundle();
                        Note note = new Note(titleInput.getText().toString(), noteInput.getText().toString(), " ");
                        textSend.putSerializable("EditActivity", note);
                        setResult(0, intent);
                        intent.putExtras(textSend);
                        finish();
                    }
                });
                builder.setNegativeButton("No", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        finish();
                    }
                });
                builder.show();
            }
            else {
                finish();
            }
        }
    }



    /*
    doReturnData() also provided on D2L: definition below taken from D2L
     */
    /*
    public void doReturnData(View v) {
        String d1Text = titleInput.getText().toString();
        String d2Text = noteInput.getText().toString();

        DataHolder dh = new DataHolder(d1Text, d2Text, d3Text);

        Intent data = new Intent(); // Used to hold results data to be returned to original activity
        data.putExtra(extraName, dh); // Better be Serializable!
        setResult(RESULT_OK, data);
        finish(); // This closes the current activity, returning us to the original activity

    }

    */

    /*
    Two functions I have taken from the first Assignment, modified them so that they don't
    rely on a built in save method, but on their own calls to the id's present in the xml file
     */
    @Override
    protected void onSaveInstanceState(Bundle outState) {

        outState.putString("TITLE", titleInput.getText().toString());
        outState.putString("NOTE", noteInput.getText().toString());

        // Call super last
        super.onSaveInstanceState(outState);
    }

    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState) {
        // Call super first
        super.onRestoreInstanceState(savedInstanceState);

        titleInput.setText(savedInstanceState.getString("TITLE"));
        noteInput.setText(savedInstanceState.getString("NOTE"));
    }



}