/*
John William Lynch
CSC 472 : Assignment 2
Multi Notes Program
Main Activity
 */

package com.example.a472_a2;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.ArrayList;
import android.text.method.ScrollingMovementMethod;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

public class MainActivity extends AppCompatActivity implements View.OnClickListener, View.OnLongClickListener {

    private static final String TAG = "MainActivity";

    //Create an array list of type notes:
    private ArrayList<Note> noteArrayList = new ArrayList<>();
    private RecyclerView recyclerView;
    private SwipeRefreshLayout swiper;
    private NoteAdapter noteAdapter;
    //Create String to Int variables that will determine what action we want to occur:
    private static final int newNote = 0;
    private static final int editNote = 1;


    /*
    Basic on create method code that starts our app
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        /*
        Set title of activity using set Title, and also
        implement way to get the amount of notes to add the notes to the
        title: ie Multi Note (0), or Multi Note (1)
         */
        setTitle("Multi Notes " + "(" + noteArrayList.size() + ")");

        /*
        Create all views/adapaters based upon lectures and provided apps:
         */
        recyclerView = findViewById(R.id.mainRecyclerView);
        noteAdapter = new NoteAdapter(noteArrayList, this);
        recyclerView.setAdapter(noteAdapter);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));


        }


        /*
        Code for doRefresh taken from gradle
         */
    private void doRefresh() {
        Collections.shuffle(noteArrayList);
        noteAdapter.notifyDataSetChanged();
        swiper.setRefreshing(false);
        Toast.makeText(this, "List content shuffled", Toast.LENGTH_SHORT).show();
    }

    /*
    Code that creates the action_menu so that icons can be displayed/selected
     */
    public boolean onCreateOptionsMenu(Menu menu){
        Log.d(TAG, "onCreate: Created the options menu ");
        getMenuInflater().inflate(R.menu.action_menu, menu);
        return true;
    }

    //My onPause call to write out to the json file:
    @Override
    protected  void onPause(){
        super.onPause();
        doWrite();
    }

    //A small helper function that simply returns the
    //date in the format specifed on the D2L PDf
    public String returnDate(){
        Calendar calendar = Calendar.getInstance();
        //The below piece of code formats it so that it shows both the day in week form and then also
        //the other bits of info needed
        SimpleDateFormat format = new SimpleDateFormat("EEE,MMM d, hh:mm aaa");
        String finalDate = format.format(calendar.getTime());
        return finalDate;
    }

    /*
    Below is the scenario where the information button is selected.
    Will do a switch case to make it so that the user then goes into the
    information screen area:
    Due to onClick() causing errors whenever a user will select it, I added the
    start activities to this boolean scenario alongside adding it to the manifest.xml file
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        switch (item.getItemId()) {
            //The user selects the ? Icon and is taken to copyright information
            //Because this just takes the user to a small screen, no need to wait for result:
            case R.id.informationIcon:
                Intent intent = new Intent ( this, AboutActivity.class);
                this.startActivity(intent);
                return true;
            //Else, the user may select the add Item icon + icon, and then we will need to bring
            //up the new activity to add in an item:
            //Make sure we use the WAIT FOR ACTIVITY code so that we can get results and then
            //edit those results
            case R.id.addItemIcon:
                Intent intent2 = new Intent (this, EditActivity.class);
                //Send request for new code ie : 0
                this.startActivityForResult(intent2, 0);
                return true;

        }
        return super.onOptionsItemSelected(item);
    }

    /*
    Small helper function to help grab arrayList index for later use:
     */
    private int grabIndex(String str){
        for (int i = 0; i < noteArrayList.size(); i++){
            if (noteArrayList.get(i).equals(str)){
                return i;
            }
        }
        return 0;
    }

    /*
    The code we operate on once we receive items back from the editActivity and determine
    how to build out our strings:
     */

    @SuppressLint("MissingSuperCall")
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data){
        //If a new note is returned:
        if (data == null){
            Toast.makeText(getApplicationContext(), "No Information Added...Please Input a Note Title", Toast.LENGTH_SHORT).show();
            return;
        }
        //Received a new Note
        if (requestCode == 0){
            Note n = (Note) data.getSerializableExtra("EditActivity");
            n.setTimestamp(returnDate());
            noteArrayList.add(0, n);
            setTitle("Multi Notes " + "(" + noteArrayList.size() + ")");
            noteAdapter.notifyDataSetChanged();
        }
        //Received request to edit existing note:
        else if (requestCode == 1){
            Note n = (Note) data.getSerializableExtra("EditActivity");
            noteArrayList.remove(grabIndex(n.getTitle()));
            n.setTimestamp(returnDate());
            noteArrayList.add(0, n);
            setTitle("Multi Notes " + "(" + noteArrayList.size() + ")");
            noteAdapter.notifyDataSetChanged();
        }
    }


    /*
    Below are the onClick/ onLongClick functions that occur once notes are created.
    A single click will bring the user back to edit the note
    A long click will delete the note:
     */
    public void onClick(View v) {  // click listener called by ViewHolder clicks

        final int pos = recyclerView.getChildLayoutPosition(v);
        Note m = noteArrayList.get(pos);
        Intent intent = new Intent (MainActivity.this, EditActivity.class);
        intent.putExtra("MainActivity", m);
        startActivityForResult(intent, 1);

    }


    /*
    On this long click, we want to prompt the user to see if they want to delete the item they
    are long clicking on:
     */
    public boolean onLongClick(View v) {  // long click listener called by ViewHolder long clicks
        final int pos = recyclerView.getChildLayoutPosition(v);
        Note m = noteArrayList.get(pos);
        String dialogChoice = "Do you want to delete this Note: " + m.getTitle() + " ?";
        AlertDialog.Builder builder = new AlertDialog.Builder(this);

        builder.setPositiveButton("YES", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                noteArrayList.remove(pos);
                setTitle("Multi Notes " + "(" + noteArrayList.size() + ")");
                noteAdapter.notifyDataSetChanged();
                dialog.dismiss();
            }
        });

    //Else the user has selected no and doesn't want to delete
        builder.setNegativeButton("CANCEL", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                dialog.dismiss();
            }
        });
        builder.setMessage(dialogChoice);
        builder.show();
        return false;

    }

    /*
    We want to also do some items in the background using async methods to read/write to our json files:
    the following two functions do that for us:

     */

    class MyAsyncTask extends AsyncTask<String, String, String> {
        @Override
        protected  void onPostExecute(String s){
            super.onPostExecute(s);
        }

        //Do in background that is required by async tasks:
        protected  String doInBackground(String... params){
            try {
                doRead();
            }
            catch (Exception error){
                error.printStackTrace();
            }
            return null;
        }
    }

    /*
    Code taken from SCRATCH code provided on D2L
    This code helps create a json file that writes out items to mydata.txt
     */

    public void doWrite() {

        JSONArray jsonArray = new JSONArray();

        for (Note n : noteArrayList) {
            try {
                JSONObject noteJSON = new JSONObject();
                noteJSON.put("titleText", n.getTitle());
                noteJSON.put("contentText", n.getContent());
                noteJSON.put("time", n.getTimestamp());
                jsonArray.put(noteJSON);
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
        String jsonText = jsonArray.toString();

        Log.d(TAG, "doWrite: " + jsonText);

        try {
            OutputStreamWriter outputStreamWriter =
                    new OutputStreamWriter(
                            openFileOutput("mydata.txt", Context.MODE_PRIVATE)
                    );

            outputStreamWriter.write(jsonText);
            outputStreamWriter.close();
            Toast.makeText(this, "Finished Writing!", Toast.LENGTH_LONG).show();
        }
        catch (IOException e) {
            Log.d(TAG, "doWrite: File write failed: " + e.toString());
            Toast.makeText(this, "File write failed: " + e.getMessage(), Toast.LENGTH_LONG).show();
        }
    }

    /*
    doRead for json file that is taken from the scratch document provided on d2l
     */


    public void doRead() {

        noteArrayList.clear();
        try {
            InputStream inputStream = openFileInput("mydata.txt");

            if ( inputStream != null ) {
                InputStreamReader inputStreamReader = new InputStreamReader(inputStream);
                BufferedReader bufferedReader = new BufferedReader(inputStreamReader);

                String receiveString = "";
                StringBuilder stringBuilder = new StringBuilder();

                while ( (receiveString = bufferedReader.readLine()) != null ) {
                    stringBuilder.append(receiveString);
                }

                inputStream.close();
                Toast.makeText(this, "TIME: " + stringBuilder.toString(), Toast.LENGTH_LONG).show();

                String jsonText = stringBuilder.toString();

                try {
                    JSONArray jsonArray = new JSONArray(jsonText);
                    Log.d(TAG, "doRead: " + jsonArray.length());

                    for (int i = 0; i < jsonArray.length(); i++) {
                        JSONObject jsonObject = jsonArray.getJSONObject(i);
                        String title = jsonObject.getString("titleText");
                        String content = jsonObject.getString("contentText");
                        String time = returnDate();
                        Note n = new Note(title, content, time);
                        noteArrayList.add(n);
                    }

                    Log.d(TAG, "doRead: " + noteArrayList);
                } catch (JSONException e) {
                    e.printStackTrace();
                }

            }
        }

        catch (FileNotFoundException e) {
            Log.d(TAG, "doRead: File not found: \" + e.toString()");
        } catch (IOException e) {
            Log.d(TAG, "doRead: Can not read file: " + e.toString());
        }

    }


}
